//-----------------------------------------------------------------------------
#ifndef TLILibImportsH
#define TLILibImportsH TLILibImportsH
//-----------------------------------------------------------------------------
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>

#ifdef __cplusplus
	extern "C" {
#endif

typedef void* MVTLI_HANDLE;
typedef void* MVTLI_INTERFACE_HANDLE;
typedef void* MVTLI_DEVICE_HANDLE;

enum INFO_DATATYPE
{
	INFO_DATATYPE_UNKNOWN     = 0,                  ///< Unknown data type
	INFO_DATATYPE_STRING      = 1,                  ///< 0-terminated C string (ASCII encoded).
	INFO_DATATYPE_STRINGLIST  = 2,                  ///< Concatenated INFO_DATATYPE_STRING list. End of list is signaled with an additional 0.
	INFO_DATATYPE_INT16       = 3,                  ///< Signed 16 bit integer.
	INFO_DATATYPE_UINT16      = 4,                  ///< unsigned 16 bit integer
	INFO_DATATYPE_INT32       = 5,                  ///< signed 32 bit integer
	INFO_DATATYPE_UINT32      = 6,                  ///< unsigned 32 bit integer
	INFO_DATATYPE_INT64       = 7,                  ///< signed 64 bit integer
	INFO_DATATYPE_UINT64      = 8,                  ///< unsigned 64 bit integer
	INFO_DATATYPE_FLOAT64     = 9,                  ///< signed 64 bit floating point number.
	INFO_DATATYPE_PTR         = 10,                 ///< Pointer type (void*). Size is platform dependent (32 bit on 32 bit platforms).
	INFO_DATATYPE_BOOL8       = 11,                 ///< Boolean value occupying 8 bit. 0 for false and anything for true.
	INFO_DATATYPE_SIZET       = 12,                 ///< Platform dependent unsigned integer (32 bit on 32 bit platforms).
	INFO_DATATYPE_BUFFER      = 13,                 ///< Like a INFO_DATATYPE_STRING but with arbitrary data and no 0 termination.
	INFO_DATATYPE_CUSTOM_ID   = 1000                ///< Starting value for custom IDs. 
};

enum INTERFACE_INFO_CMD
{
	INTERFACE_INFO_ID           = 0,
	INTERFACE_INFO_DISPLAYNAME  = 1,
	INTERFACE_INFO_TLTYPE       = 2,
	INTERFACE_INFO_CUSTOM_ID    = 1000,
	INTERFACE_INFO_MAC_STRING = INTERFACE_INFO_CUSTOM_ID,
	INTERFACE_INFO_MAC,
	INTERFACE_INFO_NETMASK_STRING,
	INTERFACE_INFO_NETMASK
};

enum DEVICE_INFO_CMD
{
	DEVICE_INFO_ID              = 0,                         ///< STRING      Unique ID of the device.
	DEVICE_INFO_VENDOR          = 1,                         ///< STRING      Device vendor name.
	DEVICE_INFO_MODEL           = 2,                         ///< STRING      Device model name.
	DEVICE_INFO_TLTYPE          = 3,                         ///< STRING      Transport layer technologies that are supported.
	DEVICE_INFO_DISPLAYNAME     = 4,                         ///< STRING      String containing a display name for the device ( including a unique id )   
	DEVICE_INFO_ACCESS_STATUS   = 5,                         ///< INT32       Gets the access mode the GenTL Producer has on the opened device. (DEVICE_ACCESS_STATUS enumeration value).
	DEVICE_INFO_CUSTOM_ID       = 1000,                      ///< Starting value for GenTL Producer custom IDs.
	DEVICE_INFO_ADAPTERNAME     = DEVICE_INFO_CUSTOM_ID,     ///< STRING      String with the GUID of the Interface adapter, this is equal to DEVICE_INFO_CUSTOM_CMD in the used version of the TLI headers
	DEVICE_INFO_ADAPTERDESCRIPTION,							///< STRING      String with the Description of the Interface adapter
	DEVICE_INFO_MANUFACTURER_SPECIFIC_INFO,					///< STRING      String containing the manufacturer specific info
	DEVICE_INFO_GVCP_MESSAGE_TIMEOUT,						///< UINT32_T    Timeout of the message channel in ms
	DEVICE_INFO_LOGMSGWRITER_NAME,							///< STRING      String containing the name of the log message writer
	DEVICE_INFO_VERSION,									///< STRING      String containing the device version register
	DEVICE_INFO_SUPPORT_USER_DEFINED_NAME,					///< 8 bit bool
	DEVICE_INFO_PERSISTENT_IP_STRING,
	DEVICE_INFO_PERSISTENT_NETMASK_STRING,
	DEVICE_INFO_PERSISTENT_DEFAULT_GATEWAY_STRING,
	DEVICE_INFO_CURRENT_NETMASK_STRING,
	DEVICE_INFO_CURRENT_DEFAULT_GATEWAY_STRING,
	DEVICE_INFO_INTERFACE_COUNT,
	DEVICE_INFO_AUTONEG_OPTIMAL_SCPS_VALUE,
	DEVICE_INFO_DATA_STREAM_COUNT,
	DEVICE_INFO_IP_STRING,
	DEVICE_INFO_MAC_STRING,
	DEVICE_INFO_SERIALNUMBER,
	DEVICE_INFO_USERNAME,
	DEVICE_INFO_MAC,
	DEVICE_INFO_IP,
	DEVICE_INFO_SUBNET,
	DEVICE_INFO_GATEWAY,
	DEVICE_INFO_SUPPORT_IP_LLA,
	DEVICE_INFO_SUPPORT_IP_DHCP,
	DEVICE_INFO_SUPPORT_IP_PERSISTENT,
	DEVICE_INFO_CURRENT_IP_LLA,
	DEVICE_INFO_CURRENT_IP_DHCP,
	DEVICE_INFO_CURRENT_IP_PERSISTENT
};

enum DEVICE_ACCESS_FLAGS
{
	DEVICE_ACCESS_UNKNOWN   = 0,         ///< Not used in a command. Can be used to initialize a variable to query that information.
	DEVICE_ACCESS_NONE      = 1,         ///< This either means that the device is not open because it was not opened before or the access to it was denied.
	DEVICE_ACCESS_READONLY  = 2,         ///< Open the device read only. All Port functions can only read from the device.
	DEVICE_ACCESS_CONTROL   = 3,         ///< Open the device in a way that other hosts/processes can have read only access to the device. Device access level is read/write for this process.
	DEVICE_ACCESS_EXCLUSIVE = 4,         ///< Open the device in a way that only this host/process can have access to the device. Device access level is read/write for this process.
	DEVICE_ACCESS_CUSTOM_ID = 1000,      ///<  Starting value for GenTL Producer custom IDs. 
};

int DMR_CALL TLOpen                          ( MVTLI_HANDLE* phTL );
int DMR_CALL TLClose                         ( MVTLI_HANDLE hTL );
int DMR_CALL TLUpdateInterfaceList           ( MVTLI_HANDLE hTL, char* bHasChanged, int64_type timeout );
int DMR_CALL TLGetNumInterfaces              ( MVTLI_HANDLE hTL, unsigned int* cnt );
int DMR_CALL TLGetInterfaceID                ( MVTLI_HANDLE hTL, unsigned int iIndex, char* psName, size_t* iSize );
int DMR_CALL TLOpenInterface                 ( MVTLI_HANDLE hTL, const char *sInterfaceName, MVTLI_INTERFACE_HANDLE* phInterface );
int DMR_CALL IFClose                         ( MVTLI_INTERFACE_HANDLE hInterface );
int DMR_CALL IFGetNumDevices                 ( MVTLI_INTERFACE_HANDLE hInterface, unsigned int* pNumDevices );
int DMR_CALL IFGetDeviceID                   ( MVTLI_INTERFACE_HANDLE hInterface, unsigned int index, char * psName, size_t* pSize );
int DMR_CALL IFGetInfo                       ( MVTLI_INTERFACE_HANDLE hInterface, INTERFACE_INFO_CMD iInfoCmd, INFO_DATATYPE* piType, void* pBuffer, size_t* piSize );
int DMR_CALL IFUpdateDeviceList              ( MVTLI_INTERFACE_HANDLE hInterface, char* bHasChanged, int64_type timeout );
int DMR_CALL IFGetDeviceInfo                 ( MVTLI_INTERFACE_HANDLE hInterface, const char* pDevName, DEVICE_INFO_CMD iID, INFO_DATATYPE* pType, void* pInfo, size_t* pSize );
int DMR_CALL IFOpenDevice                    ( MVTLI_INTERFACE_HANDLE hInterface, const char* pDevName, DEVICE_ACCESS_FLAGS iOpenFlags, MVTLI_DEVICE_HANDLE* pDev );
int DMR_CALL TLIMV_IFGetDeviceInterfaceInfo  ( MVTLI_INTERFACE_HANDLE hInterface, const char* pDevName, unsigned int interfaceIndex, DEVICE_INFO_CMD iCmd, INFO_DATATYPE *pType, void* pBuf, size_t* pSize );
int DMR_CALL TLIMV_DevSetInterfaceParam      ( MVTLI_DEVICE_HANDLE hDev, unsigned int interfaceIndex, DEVICE_INFO_CMD iCmd, const void* pBuf, size_t bufSize );
int DMR_CALL TLIMV_DevSetParam               ( MVTLI_DEVICE_HANDLE hDev, DEVICE_INFO_CMD iCmd, const void* pBuf, size_t bufSize );
int DMR_CALL DevClose                        ( MVTLI_DEVICE_HANDLE hDev );
int DMR_CALL TLIMV_MACFromSerial             ( const char* pSerial, char* pBuf, size_t* pBufSize );
int DMR_CALL TLIMV_IsValidIPv4Address        ( const char* pData );
int DMR_CALL TLIMV_DoAdressesMatch           ( const char* pIP1, const char* pNM1, const char* pIP2, const char* pNM2 );
int DMR_CALL TLIMV_ForceIP                   ( const char* pMACAddress, const char* pNewDeviceIPAddress, const char* pStaticSubnetMask, const char* pStaticDefaultGateway, const char* pAdapterIPAddress, unsigned int timeout_ms );

#ifdef __cplusplus
	}
#endif

#endif // TLILibImportsH
